"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _chai = require("chai");

var _InteractionType = _interopRequireDefault(require("quiz-interactions/lib/records/InteractionType"));

var _constants = require("@instructure/quiz-common/lib/constants");

var _Fetcher = _interopRequireDefault(require("../../../common/util/Fetcher"));

var _callHelpers = require("../../../common/util/callHelpers");

var _items = require("../items");

var _items2 = require("../../../common/actions/items");

var _Item = _interopRequireDefault(require("../../../common/records/Item"));

var _fakeFetch = _interopRequireDefault(require("../../../../tests/util/fakeFetch"));

var _jsonify = require("../../../common/util/jsonify");

describe('items api actions creators', function () {
  var dispatchStub = sinon.stub();
  var getStateStub = sinon.stub();
  var fetchStub = sinon.stub();
  useFakeFetching({
    dispatchStub: dispatchStub,
    getStateStub: getStateStub,
    fetchStub: fetchStub
  });
  var quizId = '22';
  var position = 3;
  var item = new _Item.default({
    id: '22',
    itemBody: 'itemBody',
    title: 'the title',
    position: position,
    properties: {},
    scoringData: 'answer',
    interactionType: 3,
    outcomeAlignmentSetGuid: 'ce39d262-b8f6-4650-91c1-5a76b2c18f70',
    interactionData: 'foo',
    stimulusId: '4',
    calculatorType: 'basic'
  });
  describe('getItem', function () {
    var itemId = '1';

    var stubbedGetItemCall = function stubbedGetItemCall() {
      var getItemThunk = (0, _items.getItem)(itemId);
      getItemThunk(dispatchStub, getStateStub);
    };

    it('should hit the right endpoint', function () {
      stubbedGetItemCall();
      (0, _chai.expect)(fetchStub.calledWith("/api/items/".concat(itemId))).to.be.true();
    });
  });
  describe('createItem', function () {
    afterEach(function () {
      item.getInteractionType.restore && item.getInteractionType.restore();
    });
    var quizEntry = {
      position: 8,
      pointsPossible: 18
    };

    var stubbedCreateItemCall = function stubbedCreateItemCall() {
      var qiData = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : quizEntry;
      var interactionType = new _InteractionType.default({
        scoringAlgorithmOptions: ['string'],
        userResponseTypeOptions: ['string'],
        scoringAlgorithmDefault: 'string'
      });
      sinon.stub(item, 'getInteractionType').returns(interactionType);
      _Fetcher.default.fakeFetch = fetchStub;
      var createItemThunk = (0, _items.createItem)(item, '3', 'quiz', 1);
      createItemThunk(dispatchStub, getStateStub);
    };

    it('should mark the create item call as started', function () {
      var createCallAction = (0, _callHelpers.startCallFor)(_constants.CREATE_ITEM_CALL);
      stubbedCreateItemCall();
      checkActionDispatched(dispatchStub, createCallAction);
    });
    it('should hit the right endpoint', function () {
      stubbedCreateItemCall();
      (0, _chai.expect)(fetchStub.calledWith('/api/quizzes/3/items')).to.be.true();
    });
    it('should serialize data properly', function () {
      stubbedCreateItemCall();
      var expectedData = JSON.stringify({
        item: {
          feedback: {},
          interaction_data: 'foo',
          interaction_type_id: 3,
          item_body: 'itemBody',
          outcome_alignment_set_guid: 'ce39d262-b8f6-4650-91c1-5a76b2c18f70',
          parent_item_id: 1,
          properties: {},
          scoring_algorithm: 'string',
          scoring_data: 'answer',
          title: 'the title',
          user_response_type: 'string',
          stimulus_id: '4',
          calculator_type: 'basic'
        }
      });
      (0, _chai.expect)(fetchStub.getCall(0).args[1].body).to.be.eql(expectedData);
    });

    var fakeGetQuizEntryActionCreator = function fakeGetQuizEntryActionCreator() {
      return {
        type: 'FAKE_GET_QUIZ_ENTRY'
      };
    };

    var fakeCreateItem = function fakeCreateItem() {
      var response = (0, _jsonify.jsoniblePromise)(item.toJS());
      var fakeFetch = (0, _fakeFetch.default)(response);
      var interactionType = new _InteractionType.default({
        scoringAlgorithmOptions: ['string'],
        userResponseTypeOptions: ['string']
      });
      sinon.stub(item, 'getInteractionType').returns(interactionType);
      _Fetcher.default.fakeFetch = fakeFetch;
      var createItemThunk = (0, _items.createItem)(item, {
        position: 8
      }, {
        getQuizEntriesActionCreator: fakeGetQuizEntryActionCreator
      });
      return createItemThunk(dispatchStub, getStateStub);
    };

    it('should mark the create call as a success on return', function (done) {
      var successfulCreate = (0, _callHelpers.successCallFor)(_constants.CREATE_ITEM_CALL);
      var fetchPromise = fakeCreateItem();
      fetchPromise.then(function () {
        checkActionDispatched(dispatchStub, successfulCreate);
        done();
      });
    });
    it('should try to add the item to the store', function (done) {
      var fetchPromise = fakeCreateItem();
      var handleItemsAction = (0, _items2.addItems)([item.toJS()]);
      fetchPromise.then(function () {
        checkActionDispatched(dispatchStub, handleItemsAction);
        done();
      });
    });
  });
  describe('updateItem', function () {
    var itemData = {
      id: '2',
      itemBody: 'new body',
      title: 'a new title',
      scoringData: '22',
      scoringAlgorithm: 'UpdatedAlgo',
      interactionData: {},
      properties: {},
      outcomeAlignmentSetGuid: '0281acd1-431a-4c94-a9cd-ba3429b887e6',
      calculatorType: 'basic'
    };

    var stubbedUpdateItemCall = function stubbedUpdateItemCall() {
      _Fetcher.default.fakeFetch = fetchStub;
      var item = new _Item.default(itemData);
      var updateItemThunk = (0, _items.updateItem)(item, quizId);
      updateItemThunk(dispatchStub, getStateStub);
    };

    it('should hit the right endpoint', function () {
      stubbedUpdateItemCall();
      (0, _chai.expect)(fetchStub.calledWith("/api/quizzes/".concat(quizId, "/items/2"))).to.be.true();
    });
    it('should mark the update item call as started', function () {
      stubbedUpdateItemCall();
      var updateCallStartAction = (0, _callHelpers.startCallFor)(_constants.UPDATE_ITEM_CALL);
      checkActionDispatched(dispatchStub, updateCallStartAction);
    });
    it('should serialize data properly', function () {
      stubbedUpdateItemCall();
      var expectedData = JSON.stringify({
        item: {
          item_body: 'new body',
          title: 'a new title',
          scoring_data: '22',
          scoring_algorithm: 'UpdatedAlgo',
          interaction_data: {},
          properties: {},
          outcome_alignment_set_guid: '0281acd1-431a-4c94-a9cd-ba3429b887e6',
          feedback: {},
          calculator_type: 'basic'
        }
      });
      (0, _chai.expect)(fetchStub.getCall(0).args[1].body).to.be.eql(expectedData);
    });

    var fakeUpdateItem = function fakeUpdateItem() {
      var response = (0, _jsonify.jsoniblePromise)(itemData);
      var fakeFetch = (0, _fakeFetch.default)(response);
      _Fetcher.default.fakeFetch = fakeFetch;
      var item = new _Item.default(itemData);
      var updateItemThunk = (0, _items.updateItem)(item, quizId);
      return updateItemThunk(dispatchStub, getStateStub);
    };

    it('should mark the update call as a success on return', function (done) {
      var fetchPromise = fakeUpdateItem();
      var successfulUpdate = (0, _callHelpers.successCallFor)(_constants.UPDATE_ITEM_CALL);
      fetchPromise.then(function () {
        checkActionDispatched(dispatchStub, successfulUpdate);
        done();
      });
    });
    it('should try to update the item in the store', function (done) {
      var fetchPromise = fakeUpdateItem();
      var handleItemAction = (0, _items2.addItems)([itemData]);
      fetchPromise.then(function () {
        checkActionDispatched(dispatchStub, handleItemAction);
        done();
      });
    });
  });
});